<?php

/*
 * @package RS-MINI
 * @copyright (c) 2015 Alexey Glumov aka Rio-Shaman (support@rio-shaman.ru)
 * @license GNU General Public License version 2; see LICENSE.txt
 *
 */
 
namespace core;

if(!defined('RS-MINI')) die();

/*
 * abstractmodel v 1.0
 * 
 * абстрактный класс для моделей
 *
 */

abstract class abstractmodel
{
    
	/*
     * хранит значение посланные в метод __call()
     *
     * @var    - массив
     * @access - private
     */
    
    private $call = array();
    
	/*
     * создание объекта
     * функция get_called_class() позволяет получить полное имя текущего
     * класса (тоесть того класса из которого была запущена данная функция)
     * если вызываемая модель будет лежать в папке
     * "/project/modules/pub/test/model/a.class.php", сам класс будет
     * называться "a" а неймспейс у класса будет "pub\test\model" то функция
     * get_called_class() выдаст вот такой результат: "pub\test\model\a"
     * 
     * нам остается только вызвать полученную строчку через спец слово new
     * тем самым поднять объект вызываемой модели
     * 
     * например для того что бы поднять объект класса "a" достаточно написать
     * вот так: 
     * $modelA = \pub\test\model\a::create();
     * 
     * @access - public
     *
     */
	
	public static function create()
	{
        // получаем полное имя вызываемого класса
        $who = get_called_class();
		
		// поднимаем объект этого класса
        return new $who();
	}
	
    /*
     * метод позволяем вызвать несуществующие методы класса
     * данный метод обрабатывает не все подрят а только
     * те методы которые начинаются на get (получить) или set (записать).
     * 
     * суть проста, когда разработчик юзает метод setЧтоНибудь(значение)
     * в свойстве call создается ключ массива "чтонибудь" (имя метода
     * приводится к нижнему регистру а "get" из имени вырезается) с значением
     * "значение"
     * 
     * когда разработчик юзает метод getЧтоНибудь(), метод __call из свойства
     * call вытаскивает значение ключа "чтонибудь" (имя метода приводится к
     * нижнему регистру а "set" из имени вырезается). если нет ключа "чтонибудь"
     * метод __call (в примере getЧтоНибудь()) вернет NULL
     * 
     * метод необходим что бы из контроллеров можно было передать в модель
     * значения для работы этой модели. Например если у нас есть некая модель "a"
     * которая призвана что-то вставить в базу данных, и есть таблица БД с полем
     * name, то контроллер, для того что бы что-то вставить в поле name через
     * модель "a", может передать значения (в модель) для этого поля следующим образом:
     * (метод insert() в модели отвечает за вставку строк в БД)
     * 
     * a->setName('имя')->insert();
     * 
     * В методе insert() модели "a" имя можно будет получить вот так:
     * 
     * $name = $this->getName();
     * 
     * 
     * 
     * @access - public
     * @return - сам себя или сохраненное значение (все зависит от
     *           того set или get метод был вызван)
     * 
     * @param string name      - имя вызываемого метода
     * @param array  arguments - присланные аргументы в вызываемый метод
     */
    
    public function __call($name, $arguments)
    {
        // ниже в комментариях приведен конкретный пример с методами getName() и setName().
        // предаваемое значение - "Алексей"
        
        // приводим имя вызываемого метода к нижнему регистру
        // в случае если вызван метод setName(), то получим setname
        // в случае если вызван метод getName(), то получим getname
        $name     = mb_strtolower($name);
        
        // делим имя вызываемого метода на части
        // в случае если вызван метод setName(), то получим set и name
        // в случае если вызван метод getName(), то получим get и name
        
        // в $action помещаем set или get
        $action   = mb_substr($name, 0, 3);
        // в $argument помещаем часть имени метода (при вызове
        // getName() или setName() частью будет - name)
        $argument = mb_substr($name, 3, mb_strlen($name)-1);
        
        // в зависимости от того что в $action
        switch($action) {
            case 'get':
                // возвращаем из свойства call значение (в нашем примере из ключа name
                // получим значение "Алексей", если это значение записали путем вызова
                // метода setName() с значением "Алексей")
                return (isset($this->call[$argument])) ? $this->call[$argument] : NULL;
                break;
            case 'set':
                // сохраняем в свойстве call значение  (в нашем примере в ключ name
                // свойства call будет передано значение "Алексей")
                $this->call[$argument] = $arguments[0];

                // возвращаем объект модели
                return $this;
                break;
        }
    }
    
    /*
     * метод поднимает объект базы данных и исполняет запрос
     * 
     * @access - protected
     * @return - объект класса sql
     * 
     * @param string sql - скуль запрос
     */
    
    protected function query($sql)
    {
        // исполняем запрос, и помещаем объект в свойство
        $obj = sql::query($sql);
        
        // запоминаем результат запроса
        $this->setNumRows($obj->numRows)
             ->setAffectedRows($obj->affectedRows)
             ->setInsertID($obj->insertID)
        ;
        
        // возвращаем объект
        return $obj;
    }
}
